Package: longreadvqs
Title: Viral Quasispecies Comparison from Long-Read Sequencing Data
Version: 0.1.5
Authors@R: person("Nakarin", "Pamornchainavakul", email = "pamornakarin@gmail.com", role = c("aut", "cre"), comment = c(ORCID = "0000-0003-0378-0316"))
Description: Performs variety of viral quasispecies diversity analyses [see Pamornchainavakul et al. (2026) <doi:10.1371/journal.pcbi.1014208>] based on long-read sequence alignment. Main functions include 1) sequencing error and other noise minimization and read sampling, 2) Single nucleotide variant (SNV) profiles comparison, and 3) viral quasispecies profiles comparison and visualization. 
License: GPL-3
URL: https://github.com/NakarinP/longreadvqs
BugReports: https://github.com/NakarinP/longreadvqs/issues
Encoding: UTF-8
RoxygenNote: 7.3.3
biocViews:
Imports: ape, Biostrings, cowplot, dplyr, ggplot2, ggpubr, grDevices,
        IRanges, magrittr, methods, plyr, purrr, pwalign, QSutils,
        RColorBrewer, reshape2, scales, seqinr, stats, stringdist,
        stringr, tibble, tidyr
Depends: R (>= 4.4)
Suggests: knitr, rmarkdown, testthat (>= 3.0.0)
VignetteBuilder: knitr
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-04-28 23:08:31 UTC; nakarinpamornchainavakul
Author: Nakarin Pamornchainavakul [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-0378-0316>)
Maintainer: Nakarin Pamornchainavakul <pamornakarin@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-28 23:30:02 UTC
Built: R 4.6.0; ; 2026-04-29 01:48:47 UTC; unix
