# Generated by roxygen2: do not edit by hand

export("%>%")
export(AAcompare)
export(filtfast)
export(gapremove)
export(otucompare)
export(pctopt)
export(snvcompare)
export(vqsassess)
export(vqscompare)
export(vqscustompct)
export(vqsout)
export(vqsresub)
export(vqssub)
import(QSutils)
import(RColorBrewer)
import(dplyr)
import(ggplot2)
import(scales)
import(stringr)
import(tidyr)
importFrom(Biostrings,AAStringSet)
importFrom(Biostrings,DNAStringSet)
importFrom(Biostrings,GENETIC_CODE)
importFrom(Biostrings,nmismatch)
importFrom(Biostrings,readDNAStringSet)
importFrom(Biostrings,width)
importFrom(IRanges,Views)
importFrom(ape,as.AAbin)
importFrom(ape,as.DNAbin)
importFrom(ape,dist.aa)
importFrom(ape,dist.dna)
importFrom(cowplot,get_legend)
importFrom(cowplot,plot_grid)
importFrom(ggpubr,ggscatter)
importFrom(magrittr,"%>%")
importFrom(magrittr,set_colnames)
importFrom(methods,is)
importFrom(plyr,rbind.fill)
importFrom(pwalign,pairwiseAlignment)
importFrom(reshape2,melt)
importFrom(seqinr,as.alignment)
importFrom(seqinr,consensus)
importFrom(seqinr,read.alignment)
importFrom(stats,aggregate)
importFrom(stats,as.dist)
importFrom(stats,cmdscale)
importFrom(stats,kmeans)
importFrom(stats,quantile)
importFrom(stringdist,stringsim)
importFrom(utils,write.table)
