#' @keywords internal
"_PACKAGE"

## usethis namespace: start
#' @importFrom Biostrings AAStringSet
#' @importFrom Biostrings GENETIC_CODE
#' @importFrom IRanges Views
#' @importFrom methods is
## usethis namespace: end
NULL
